@@ -184,9 +184,7 @@ extern "C" {
 #include "nsString.h"
 #include "nsIURI.h"
 #include "nsIURL.h"
-#include "nsIMIMEInfo.h"
 #include "nsIChannel.h"
-#include "nsIFTPChannel.h"
 #include "nsILocalFile.h"
 #include "nsIPrefService.h"
 #include "nsIDOMWindow.h"
@@ -231,8 +229,7 @@ NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
 GContentHandler::GContentHandler() : mUri(nsnull),
 				     mMimeType(nsnull),
-				     mDownloadCanceled(PR_FALSE),
-				     mHelperProgress(PR_FALSE)
+				     mDownloadCanceled(PR_FALSE)
 {
 	NS_INIT_ISUPPORTS();
 	/* member initializers and constructor code */
@@ -249,10 +246,20 @@ GContentHandler::~GContentHandler()
 // begin nsIHelperAppLauncher impl
 ////////////////////////////////////////////////////////////////////////////////
 
+#if MOZILLA_SNAPSHOT > 8
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
+NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
+				    nsISupports *aContext,
+				    PRBool aForced)
+#else
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 				    nsISupports *aContext)
+#endif
 {
+	/* aForced reflects if the content being sent is normally viewable
+	 * in mozilla or not. That fact doesn't affect us, so ignore it
+         */
+
 	nsresult rv;
 
 	mLauncher = aLauncher;
@@ -281,8 +288,11 @@ NS_IMETHODIMP GContentHandler::
 	nsCOMPtr<nsILocalFile> saveDir;
 	char *dirName;
 	
-	/* FIXME persist download dir */
-	dirName = g_strdup (g_get_home_dir());
+	dirName = eel_gconf_get_string (CONF_STATE_DOWNLOADING_DIR);
+	if (dirName == NULL)
+	{
+		dirName = g_strdup (g_get_home_dir());
+	}
 
 	saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
 	saveDir->InitWithPath (NS_ConvertUTF8toUCS2(dirName));
@@ -292,45 +302,52 @@ NS_IMETHODIMP GContentHandler::
 
 	PRInt16 okToSave = nsIFilePicker::returnCancel;
 
-	if (okToSave == nsIFilePicker::returnCancel)
+	nsCOMPtr<nsIFilePicker> filePicker =
+				do_CreateInstance (G_FILEPICKER_CONTRACTID);
+
+	const nsAString &title = NS_ConvertUTF8toUCS2(_("Select the destination filename"));
+
+	filePicker->Init (windowInternal,
+			   PromiseFlatString(title).get(), 
+			   nsIFilePicker::modeSave);
+	filePicker->SetDefaultString (aDefaultFile);
+	filePicker->SetDisplayDirectory (saveDir);
+
+	filePicker->Show (&okToSave);
+
+	if (okToSave == nsIFilePicker::returnOK)
 	{
-		nsCOMPtr<nsIFilePicker> filePicker =
-					do_CreateInstance (G_FILEPICKER_CONTRACTID);
+		filePicker->GetFile (getter_AddRefs(saveFile));
 
-		const nsAString &title = NS_ConvertUTF8toUCS2(_("Select the destination filename"));
+		nsString uFileName;
+		saveFile->GetPath(uFileName);
+		const nsCString &aFileName = NS_ConvertUCS2toUTF8(uFileName);
 
-		filePicker->Init (windowInternal,
-				   PromiseFlatString(title).get(), 
-				   nsIFilePicker::modeSave);
-		filePicker->SetDefaultString (aDefaultFile);
-		filePicker->SetDisplayDirectory (saveDir);
+		char *dir = g_path_get_dirname (aFileName.get());
 
-		filePicker->Show (&okToSave);
+		eel_gconf_set_string (CONF_STATE_DOWNLOADING_DIR, dir);
+		g_free (dir);
 
-		if (okToSave == nsIFilePicker::returnOK)
-		{
-			filePicker->GetFile (getter_AddRefs(saveFile));
-		}
-	}
-
-	if (okToSave == nsIFilePicker::returnCancel)
-		return NS_ERROR_FAILURE;
-	else
-	{
 		nsCOMPtr<nsIFile> directory;
 		rv = saveFile->GetParent (getter_AddRefs(directory));
 
 		NS_IF_ADDREF (*_retval = saveFile);
 		return NS_OK;
 	}
+	else
+	{
+		return NS_ERROR_FAILURE;
+	}
 }
 
+#if MOZILLA_SNAPSHOT < 9
 /* void showProgressDialog (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 					      nsISupports *aContext)
 {
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
+#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // begin local public methods impl
@@ -393,12 +410,6 @@ NS_METHOD GContentHandler::LaunchHelperApp (void)
 	return NS_OK;
 }
 
-NS_METHOD GContentHandler::ShowHelperProgressDialog (void)
-{
-	mHelperProgress = PR_TRUE;
-	return ShowProgressDialog (mLauncher,mContext);
-}
-
 NS_METHOD GContentHandler::GetLauncher (nsIHelperAppLauncher * *_retval)
 {
 	NS_IF_ADDREF (*_retval = mLauncher);
@@ -458,7 +469,7 @@ NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;	
 
 	nsMIMEInfoHandleAction mimeInfoAction;
-	mimeInfoAction = nsIMIMEInfo::alwaysAsk;
+	mimeInfoAction = nsIMIMEInfo::useHelperApp;
 
 	if(mHelperApp->requires_terminal) //Information passing kludge!
 	{
