@@ -51,10 +51,6 @@ impl_construct (EphyDialog *dialog,
 		const EphyDialogProperty *properties,
 		const char *file,
 		const char *name);
-
-static void
-impl_destruct (EphyDialog *dialog);
-
 static GtkWidget *
 impl_get_control (EphyDialog *dialog,
                   int property_id);
@@ -161,7 +157,6 @@ ephy_dialog_class_init (EphyDialogClass *klass)
 	klass->get_value = impl_get_value;
 	klass->run = impl_run;
 	klass->show = impl_show;
-	klass->destruct = impl_destruct;
 
 	g_object_class_install_property (object_class,
                                          PROP_PARENT_WINDOW,
@@ -939,7 +934,7 @@ ephy_dialog_finalize (GObject *object)
 
 	if (dialog->priv->dialog)
 	{
-		ephy_dialog_destruct (dialog);
+		gtk_widget_destroy (dialog->priv->dialog);
 	}
 
 	free_props (dialog->priv->props);
@@ -1064,15 +1059,6 @@ init_props (const EphyDialogProperty *properties, GladeXML *gxml)
 	return props;
 }
 
-static void
-dialog_destruction_notify (EphyDialog *dialog,
-                           GObject *where_the_object_was)
-{
-	dialog->priv->dialog = NULL;
-	ephy_dialog_destruct (dialog);
-	g_object_unref (dialog);
-}
-
 static void
 dialog_destroy_cb (GtkWidget *widget, EphyDialog *dialog)
 {
@@ -1081,6 +1067,8 @@ dialog_destroy_cb (GtkWidget *widget, EphyDialog *dialog)
 	{
 		save_props (dialog->priv->props);
 	}
+
+	g_object_unref (dialog);
 }
 
 static void
@@ -1104,29 +1092,12 @@ impl_construct (EphyDialog *dialog,
 		dialog->priv->props = init_props (properties, gxml);
 	}
 
-	g_signal_connect (dialog->priv->dialog,
-			  "destroy",
-			  G_CALLBACK(dialog_destroy_cb),
-			  dialog);
+	g_signal_connect_object (dialog->priv->dialog,
+			         "destroy",
+			         G_CALLBACK(dialog_destroy_cb),
+			         dialog, 0);
 
 	g_object_unref (gxml);
-
-	g_object_weak_ref (G_OBJECT(dialog->priv->dialog),
-			   (GWeakNotify)dialog_destruction_notify,
-			   dialog);
-}
-
-static void
-impl_destruct (EphyDialog *dialog)
-{
-	if (dialog->priv->dialog)
-	{
-		g_object_weak_unref (G_OBJECT(dialog->priv->dialog),
-				     (GWeakNotify)dialog_destruction_notify,
-				     dialog);
-		gtk_widget_destroy (dialog->priv->dialog);
-		dialog->priv->dialog = NULL;
-	}
 }
 
 static GtkWidget *
@@ -1245,13 +1216,6 @@ ephy_dialog_construct (EphyDialog *dialog,
         return klass->construct (dialog, properties, file, name);
 }
 
-void
-ephy_dialog_destruct (EphyDialog *dialog)
-{
-	EphyDialogClass *klass = EPHY_DIALOG_GET_CLASS (dialog);
-        klass->destruct (dialog);
-}
-
 gint
 ephy_dialog_run (EphyDialog *dialog)
 {
@@ -1283,3 +1247,25 @@ ephy_dialog_get_value (EphyDialog *dialog,
         return klass->get_value (dialog, property_id, value);
 }
 
+void
+ephy_dialog_set_size_group (EphyDialog *dialog,
+			    int *controls_id,
+			    guint n_controls)
+{
+	GtkSizeGroup *size_group;
+	int i;
+
+	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
+	for (i = 0; i < n_controls; i++)
+	{
+		GtkWidget *widget;
+		guint id;
+
+		id = controls_id[i];
+		widget = dialog->priv->props[id].widget;
+		g_return_if_fail (GTK_IS_WIDGET (widget));
+
+                gtk_size_group_add_widget (size_group, widget);
+        }
+}
