@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.16 2003/07/14 09:51:37 xan Exp $
+ *  $Id: ephy-node-view.c,v 1.19 2003/07/27 13:10:53 mpeseng Exp $
  */
 
 #include <gtk/gtktreeselection.h>
@@ -24,6 +24,7 @@
 #include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkwindow.h>
 #include <gdk/gdkkeysyms.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 
 #include "eggtreemodelfilter.h"
 #include "ephy-node-view.h"
@@ -58,8 +59,6 @@ struct EphyNodeViewPrivate
 
 	EphyNodeFilter *filter;
 
-	EphyNode *selected_node;
-
 	GtkTargetList *drag_targets;
 
 	int default_sort_column_id;
@@ -87,8 +86,6 @@ enum
 	PROP_FILTER
 };
 
-static EphyNodeView *target_view;
-
 static GObjectClass *parent_class = NULL;
 
 static guint ephy_node_view_signals[LAST_SIGNAL] = { 0 };
@@ -310,18 +307,15 @@ drag_data_received_cb (GtkWidget *widget,
 					       x, y, &path, &pos))
 	{
 		EphyNode *node;
-		GList *src_nodes;
+		GList *uris;
 
 		node = get_node_from_path (view, path);
 
-		src_nodes = ephy_dnd_node_list_extract_nodes
-				(selection_data->data);
-
+		uris = gnome_vfs_uri_list_parse (selection_data->data);
 		g_signal_emit (G_OBJECT (view),
 			       ephy_node_view_signals[NODE_DROPPED], 0,
-			       node, src_nodes);
-
-		g_list_free (src_nodes);
+			       node, uris);
+		gnome_vfs_uri_list_free (uris);
 
 		gtk_tree_path_free (path);
 	}
@@ -388,8 +382,6 @@ ephy_node_view_selection_changed_cb (GtkTreeSelection *selection,
 	GList *list;
 	EphyNode *node = NULL;
 
-	view->priv->selected_node = NULL;
-
 	list = ephy_node_view_get_selection (view);
 	if (list)
 	{
@@ -432,7 +424,7 @@ ephy_node_view_select_node_by_key (EphyNodeView *view, GdkEventKey *event)
 	gboolean found = FALSE;
 	gchar outbuf[6];
 	gint length;
-	
+
 	length = g_unichar_to_utf8 (gdk_keyval_to_unicode (event->keyval), outbuf);
 	event_string = g_utf8_casefold (outbuf, length);
 
@@ -462,7 +454,7 @@ ephy_node_view_select_node_by_key (EphyNodeView *view, GdkEventKey *event)
 	{
 		iter = last_iter;
 	}
-	
+
 	path = gtk_tree_model_get_path (view->priv->sortmodel, &iter);
 	gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
 	gtk_tree_path_free (path);
@@ -493,44 +485,36 @@ ephy_node_view_key_press_cb (GtkTreeView *treeview,
 }
 
 static gboolean
-ephy_node_view_button_press_cb (GtkTreeView *treeview,
+ephy_node_view_button_press_cb (GtkWidget *treeview,
 			        GdkEventButton *event,
 			        EphyNodeView *view)
 {
 	GtkTreePath *path = NULL;
 	GtkTreeSelection *selection;
-	gboolean result = FALSE;
 
-	if (event->button == 3)
+	if (event->button != 3) return FALSE;
+
+	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
+					   event->x,
+					   event->y,
+					   &path,
+					   NULL, NULL, NULL))
 	{
-		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
-						   event->x,
-						   event->y,
-						   &path,
-						   NULL, NULL, NULL))
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+		if (!gtk_tree_selection_path_is_selected (selection, path))
 		{
-			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-			if (gtk_tree_selection_path_is_selected (selection, path))
-			{
-				/* We handle the event (so the view won't be
-				 * changed by the user click) because the user
-				 * clicked on an already selected element */
-				result = TRUE;
-			}
-			else
-			{
-				view->priv->selected_node =
-					get_node_from_path (view, path);
-			}
-
-			target_view = view;
-			g_signal_emit (G_OBJECT (view), ephy_node_view_signals[SHOW_POPUP], 0);
-			target_view = NULL;
-			gtk_tree_path_free (path);
+			GTK_WIDGET_CLASS (parent_class)->button_press_event (treeview, event);
 		}
+		else if (!gtk_widget_is_focus (GTK_WIDGET (treeview)))
+		{
+			gtk_widget_grab_focus (GTK_WIDGET (treeview));
+		}
+
+		g_signal_emit (G_OBJECT (view), ephy_node_view_signals[SHOW_POPUP], 0);
+		gtk_tree_path_free (path);
 	}
 
-	return result;
+	return TRUE;
 }
 
 static void
@@ -586,22 +570,6 @@ ephy_node_view_get_property (GObject *object,
 	}
 }
 
-static void
-node_from_sort_iter_cb (EphyTreeModelSort *model,
-		        GtkTreeIter *iter,
-		        void **node,
-		        EphyNodeView *view)
-{
-	GtkTreeIter filter_iter, node_iter;
-
-	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model),
-							&filter_iter, iter);
-	egg_tree_model_filter_convert_iter_to_child_iter (EGG_TREE_MODEL_FILTER (view->priv->filtermodel),
-							  &node_iter, &filter_iter);
-	*node = ephy_tree_model_node_node_from_iter
-		(EPHY_TREE_MODEL_NODE (view->priv->nodemodel), &node_iter);
-}
-
 static void
 ephy_node_view_construct (EphyNodeView *view)
 {
@@ -614,11 +582,6 @@ ephy_node_view_construct (EphyNodeView *view)
 	egg_tree_model_filter_set_visible_column (EGG_TREE_MODEL_FILTER (view->priv->filtermodel),
 						  EPHY_TREE_MODEL_NODE_COL_VISIBLE);
 	view->priv->sortmodel = ephy_tree_model_sort_new (view->priv->filtermodel);
-	g_signal_connect_object (G_OBJECT (view->priv->sortmodel),
-				 "node_from_iter",
-				 G_CALLBACK (node_from_sort_iter_cb),
-				 view,
-				 0);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (view->priv->sortmodel));
 	g_signal_connect_object (G_OBJECT (view),
 			         "button_press_event",
@@ -793,6 +756,29 @@ provide_text_weight (EphyNode *node, GValue *value, EphyNodeView *view)
 	}
 }
 
+int
+ephy_node_view_add_data_column (EphyNodeView *view,
+			        GType value_type,
+				int prop_id,
+			        EphyTreeModelNodeValueFunc func,
+				gpointer data)
+{
+	int column;
+
+	if (func)
+	{
+		column = ephy_tree_model_node_add_func_column
+			(view->priv->nodemodel, value_type, func, data);
+	}
+	else
+	{
+		column = ephy_tree_model_node_add_prop_column
+			(view->priv->nodemodel, value_type, prop_id);
+	}
+
+	return column;
+}
+
 GtkTreeViewColumn *
 ephy_node_view_add_column (EphyNodeView *view,
 			   const char  *title,
@@ -913,7 +899,6 @@ ephy_node_view_init (EphyNodeView *view)
 	view->priv = g_new0 (EphyNodeViewPrivate, 1);
 	view->priv->editable_renderer = NULL;
 	view->priv->editing = TRUE;
-	view->priv->selected_node = NULL;
 	view->priv->searchable_data_column = -1;
 
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), FALSE);
@@ -951,20 +936,12 @@ ephy_node_view_get_selection (EphyNodeView *view)
 	GList *list = NULL;
 	GtkTreeSelection *selection;
 
-	if (view->priv->selected_node)
-	{
-		list = g_list_append (list, view->priv->selected_node);
-	}
-	else
-	{
-		selection = gtk_tree_view_get_selection
-			(GTK_TREE_VIEW (view));
+	selection = gtk_tree_view_get_selection	(GTK_TREE_VIEW (view));
 
-		gtk_tree_selection_selected_foreach
+	gtk_tree_selection_selected_foreach
 			(selection,
 			 (GtkTreeSelectionForeachFunc) get_selection,
 			 (gpointer) &list);
-	}
 
 	return list;
 }
@@ -972,7 +949,7 @@ ephy_node_view_get_selection (EphyNodeView *view)
 void
 ephy_node_view_remove (EphyNodeView *view)
 {
-	GList *list;
+	GList *list, *l;
 	EphyNode *node;
 	GtkTreeIter iter, iter2;
 	GtkTreePath *path;
@@ -1009,9 +986,9 @@ ephy_node_view_remove (EphyNodeView *view)
 	}
 	gtk_tree_path_free (path);
 
-	for (; list != NULL; list = list->next)
+	for (l = list; l != NULL; l = l->next)
 	{
-		ephy_node_unref (list->data);
+		ephy_node_unref (l->data);
 	}
 
 	g_list_free (list);
@@ -1021,9 +998,14 @@ ephy_node_view_remove (EphyNodeView *view)
 	if (row_ref != NULL)
 	{
 		path = gtk_tree_row_reference_get_path (row_ref);
-		gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
+
+		if (path != NULL)
+		{
+			gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
+			gtk_tree_path_free (path);
+		}	
+
 		gtk_tree_row_reference_free (row_ref);
-		gtk_tree_path_free (path);
 	}
 }
 
@@ -1058,7 +1040,7 @@ void
 ephy_node_view_enable_drag_source (EphyNodeView *view,
 				   GtkTargetEntry *types,
 				   int n_types,
-				   guint prop_id)
+				   int column)
 {
 	GtkWidget *treeview;
 
@@ -1073,8 +1055,8 @@ ephy_node_view_enable_drag_source (EphyNodeView *view,
 						types, n_types,
 						GDK_ACTION_COPY);
 
-	ephy_tree_model_sort_set_drag_property (EPHY_TREE_MODEL_SORT (view->priv->sortmodel),
-						prop_id);
+	ephy_tree_model_sort_set_column_id (EPHY_TREE_MODEL_SORT (view->priv->sortmodel),
+					    column);
 }
 
 void
@@ -1109,18 +1091,7 @@ ephy_node_view_edit (EphyNodeView *view)
 gboolean
 ephy_node_view_is_target (EphyNodeView *view)
 {
-	if (target_view == view)
-	{
-		return TRUE;
-	}
-	else if (target_view != NULL)
-	{
-		return FALSE;
-	}
-	else
-	{
-		return gtk_widget_is_focus (GTK_WIDGET (view));
-	}
+	return gtk_widget_is_focus (GTK_WIDGET (view));
 }
 
 gboolean
@@ -1129,17 +1100,8 @@ ephy_node_view_has_selection (EphyNodeView *view, gboolean *multiple)
 	GtkTreeSelection *selection;
 	int rows;
 
-	if (view->priv->selected_node)
-	{
-		rows = 1;
-	}
-	else
-	{
-		selection = gtk_tree_view_get_selection
-			(GTK_TREE_VIEW (view));
-
-		rows = gtk_tree_selection_count_selected_rows (selection);
-	}
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+	rows = gtk_tree_selection_count_selected_rows (selection);
 
 	if (multiple)
 	{
@@ -1148,4 +1110,3 @@ ephy_node_view_has_selection (EphyNodeView *view, gboolean *multiple)
 
 	return rows > 0;
 }
-
