@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: ephy-bookmarks-editor.c,v 1.90 2003/06/23 00:34:19 chpe Exp $
+ * $Id: ephy-bookmarks-editor.c,v 1.95 2003/07/27 13:10:56 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +32,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomeui/gnome-stock-icons.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 #include <string.h>
 
 #include "ephy-bookmarks-editor.h"
@@ -55,10 +56,12 @@
 #include "ephy-gui.h"
 #include "ephy-stock-icons.h"
 #include "ephy-search-entry.h"
+#include "ephy-toolbars-model.h"
+#include "ephy-favicon-cache.h"
 
 static GtkTargetEntry topic_drag_dest_types [] =
 {
-	{ EPHY_DND_BOOKMARK_TYPE,   0, 0 }
+	{ EPHY_DND_URI_LIST_TYPE,   0, 0 }
 };
 
 static int n_topic_drag_dest_types = G_N_ELEMENTS (topic_drag_dest_types);
@@ -67,8 +70,7 @@ static GtkTargetEntry bmk_drag_types [] =
 {
         { EPHY_DND_URI_LIST_TYPE,   0, 0 },
         { EPHY_DND_TEXT_TYPE,       0, 1 },
-        { EPHY_DND_URL_TYPE,        0, 2 },
-	{ EPHY_DND_BOOKMARK_TYPE,   0, 3 }
+        { EPHY_DND_URL_TYPE,        0, 2 }
 };
 static int n_bmk_drag_types = G_N_ELEMENTS (bmk_drag_types);
 
@@ -332,7 +334,7 @@ cmd_show_in_bookmarks_bar (EggAction *action,
 	else
 	{
 		ephy_toolbars_model_remove_bookmark
-			(editor->priv->tb_model, topic, id);
+			(editor->priv->tb_model, id);
 	}
 
 	g_list_free (selection);
@@ -535,12 +537,11 @@ cmd_bookmarks_import (EggAction *action,
 	gtk_widget_show (menu);
 
 	add_bookmarks_source_menu (menu, _("Mozilla bookmarks"),
-				   ".mozilla", "bookmarks.html");
+				   MOZILLA_BOOKMARKS_DIR, "bookmarks.html");
 	add_bookmarks_source_menu (menu, _("Galeon bookmarks"),
-				   ".galeon", "bookmarks.xbel");
+				   GALEON_BOOKMARKS_DIR, "bookmarks.xbel");
 	add_bookmarks_source_menu (menu, _("Konqueror bookmarks"),
-				   ".kde/share/apps/konqueror",
-				   "bookmarks.xml");
+				   KDE_BOOKMARKS_DIR, "bookmarks.xml");
 
 	option_menu = gtk_option_menu_new ();
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
@@ -826,7 +827,7 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 
 		id = ephy_node_get_id (node);
 		show_in_bookmarks_bar = ephy_toolbars_model_has_bookmark
-			(editor->priv->tb_model, TRUE, id);
+			(editor->priv->tb_model, id);
 
 		priority = ephy_node_get_property_int
 			(node, EPHY_NODE_KEYWORD_PROP_PRIORITY);
@@ -846,7 +847,7 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 
 		id = ephy_node_get_id (node);
 		show_in_bookmarks_bar = ephy_toolbars_model_has_bookmark
-			(editor->priv->tb_model, FALSE, id);
+			(editor->priv->tb_model, id);
 
 		g_list_free (selected);
 	}
@@ -1012,16 +1013,11 @@ key_pressed_cb (EphyNodeView *view,
 		GdkEventKey *event,
 		EphyBookmarksEditor *editor)
 {
-	switch (event->keyval)
+	if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete)
 	{
-	case GDK_Delete:
-	case GDK_KP_Delete:
 		cmd_delete (NULL, editor);
 		return TRUE;
-
-	default:
-		break;
-	}
+	} 
 
 	return FALSE;
 }
@@ -1154,9 +1150,18 @@ node_dropped_cb (EphyNodeView *view, EphyNode *node,
 
 	for (l = nodes; l != NULL; l = l->next)
 	{
-		EphyNode *bmk = l->data;
+		GnomeVFSURI *uri = l->data;
+		char *url;
+		EphyNode *bmk;
 
-		ephy_bookmarks_set_keyword (editor->priv->bookmarks, node, bmk);
+		url = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+		bmk = ephy_bookmarks_find_bookmark (editor->priv->bookmarks, url);
+		g_free (url);
+
+		if (bmk)
+		{
+			ephy_bookmarks_set_keyword (editor->priv->bookmarks, node, bmk);
+		}
 	}
 }
 
@@ -1167,7 +1172,8 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	const char *icon_location;
 	GdkPixbuf *pixbuf = NULL;
 
-	cache = ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell));
+	cache = EPHY_FAVICON_CACHE
+		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
 	icon_location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_ICON);
 
@@ -1179,7 +1185,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	}
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
-	g_value_set_object (value, pixbuf);
+	g_value_set_object_take_ownership (value, pixbuf);
 }
 
 static void
@@ -1188,6 +1194,19 @@ view_selection_changed_cb (GtkWidget *view, EphyBookmarksEditor *editor)
 	ephy_bookmarks_editor_update_menu (editor);
 }
 
+static void
+provide_keyword_uri (EphyNode *node, GValue *value, gpointer data)
+{
+	EphyBookmarks *bookmarks = EPHY_BOOKMARKS_EDITOR (data)->priv->bookmarks;
+	char *uri;
+
+	uri = ephy_bookmarks_get_topic_uri (bookmarks, node);
+
+	g_value_init (value, G_TYPE_STRING);
+	g_value_set_string (value, uri);
+	g_free (uri);
+}
+
 static void
 ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 {
@@ -1200,7 +1219,7 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	EggActionGroup *action_group;
 	EggAction *action;
 	GdkPixbuf *icon;
-	int i;
+	int i, col_id;
 
 	gtk_window_set_title (GTK_WINDOW (editor), _("Bookmarks"));
 
@@ -1263,10 +1282,12 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	/* Keywords View */
 	key_view = ephy_node_view_new (node, NULL);
 	add_focus_monitor (editor, key_view);
+	col_id = ephy_node_view_add_data_column (EPHY_NODE_VIEW (key_view),
+					         G_TYPE_STRING, -1,
+						 provide_keyword_uri, editor);
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (key_view),
 					   topic_drag_types,
-				           n_topic_drag_types,
-					   -1);
+				           n_topic_drag_types, col_id);
 	ephy_node_view_enable_drag_dest (EPHY_NODE_VIEW (key_view),
 			                 topic_drag_dest_types,
 			                 n_topic_drag_dest_types);
@@ -1281,7 +1302,8 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 				   EPHY_NODE_KEYWORD_PROP_NAME,
 				   EPHY_NODE_KEYWORD_PROP_PRIORITY,
 				   EPHY_NODE_VIEW_AUTO_SORT |
-				   EPHY_NODE_VIEW_EDITABLE, NULL);
+				   EPHY_NODE_VIEW_EDITABLE |
+				   EPHY_NODE_VIEW_SEARCHABLE, NULL);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), key_view);
 	gtk_widget_set_size_request (key_view, 130, -1);
 	gtk_widget_show (key_view);
@@ -1328,14 +1350,19 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	/* Bookmarks View */
 	bm_view = ephy_node_view_new (node, editor->priv->bookmarks_filter);
 	add_focus_monitor (editor, bm_view);
+	col_id = ephy_node_view_add_data_column (EPHY_NODE_VIEW (bm_view),
+					         G_TYPE_STRING,
+					         EPHY_NODE_BMK_PROP_LOCATION,
+						 NULL, NULL);
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (bm_view),
 					   bmk_drag_types,
 				           n_bmk_drag_types,
-					   EPHY_NODE_BMK_PROP_LOCATION);
+					   col_id);
 	ephy_node_view_add_column (EPHY_NODE_VIEW (bm_view), _("Title"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_TITLE, -1,
 				   EPHY_NODE_VIEW_AUTO_SORT |
-				   EPHY_NODE_VIEW_EDITABLE,
+				   EPHY_NODE_VIEW_EDITABLE |
+				   EPHY_NODE_VIEW_SEARCHABLE,
 				   provide_favicon);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), bm_view);
 	gtk_widget_show (bm_view);
@@ -1457,7 +1484,8 @@ ephy_bookmarks_editor_init (EphyBookmarksEditor *editor)
 
 	editor->priv->props_dialogs = g_hash_table_new (g_direct_hash,
                                                         g_direct_equal);
-	editor->priv->tb_model = ephy_shell_get_toolbars_model (ephy_shell);
+	editor->priv->tb_model = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 
 	g_signal_connect (editor->priv->tb_model, "item_added",
 			  G_CALLBACK (toolbar_items_changed_cb), editor);
