@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.93 2003/06/28 17:25:42 chpe Exp $
+ *  $Id: ephy-window.c,v 1.98 2003/07/20 11:05:16 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,9 @@
 #include "ephy-encoding-menu.h"
 #include "ephy-tabs-menu.h"
 #include "ephy-stock-icons.h"
+#include "ephy-toolbars-model.h"
+#include "session.h"
+#include "ephy-favicon-cache.h"
 
 #include <string.h>
 #include <bonobo/bonobo-i18n.h>
@@ -69,10 +72,10 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 
 	/* File menu */
 	{ "FileNewWindow", N_("_New Window"), GTK_STOCK_NEW, "<control>N",
-	  N_("Create a new window"),
+	  N_("Open a new window"),
 	  G_CALLBACK (window_cmd_file_new_window), NULL },
 	{ "FileNewTab", N_("New _Tab"), EPHY_STOCK_NEW_TAB, "<control>T",
-	  N_("Create a new tab"),
+	  N_("Open a new tab"),
 	  G_CALLBACK (window_cmd_file_new_tab), NULL },
 	{ "FileOpen", N_("_Open..."), GTK_STOCK_OPEN, "<control>O",
 	  N_("Open a file"),
@@ -104,13 +107,13 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	  N_("Select the entire page"),
 	  G_CALLBACK (window_cmd_edit_select_all), NULL },
 	{ "EditFind", N_("_Find..."), GTK_STOCK_FIND, "<control>F",
-	  N_("Find a string"),
+	  N_("Find a word or phrase in the page"),
 	  G_CALLBACK (window_cmd_edit_find), NULL },
 	{ "EditFindNext", N_("Find Ne_xt"), NULL, "<control>G",
-	  N_("Find next occurrence of the string"),
+	  N_("Find next occurrence of the word or phrase"),
 	  G_CALLBACK (window_cmd_edit_find_next), NULL },
 	{ "EditFindPrev", N_("Find Pre_vious"), NULL, "<shift><control>G",
-	  N_("Find previous occurrence of the string"),
+	  N_("Find previous occurrence of the word or phrase"),
 	  G_CALLBACK (window_cmd_edit_find_prev), NULL },
 	{ "EditPersonalData", N_("P_ersonal Data"), NULL, NULL,
 	  N_("View and remove cookies and passwords"),
@@ -142,13 +145,13 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	  N_("Browse at full screen"),
 	  G_CALLBACK (window_cmd_view_fullscreen), NULL, TOGGLE_ACTION},
 	{ "ViewZoomIn", N_("Zoom _In"), GTK_STOCK_ZOOM_IN, "<control>plus",
-	  N_("Show the contents in more detail"),
+	  N_("Increase the text size"),
 	  G_CALLBACK (window_cmd_view_zoom_in), NULL },
 	{ "ViewZoomOut", N_("Zoom _Out"), GTK_STOCK_ZOOM_OUT, "<control>minus",
-	  N_("Show the contents in less detail"),
+	  N_("Decrease the text size"),
 	  G_CALLBACK (window_cmd_view_zoom_out), NULL },
 	{ "ViewZoomNormal", N_("_Normal Size"), GTK_STOCK_ZOOM_100, NULL,
-	  N_("Show the contents at the normal size"),
+	  N_("Use the normal text size"),
 	  G_CALLBACK (window_cmd_view_zoom_normal), NULL },
 	{ "ViewEncoding", N_("_Encoding"), NULL, NULL, NULL, NULL, NULL },
 	{ "ViewPageSource", N_("_Page Source"), EPHY_STOCK_VIEWSOURCE, "<control>U",
@@ -160,7 +163,7 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	  N_("Add a bookmark for the current page"),
 	  G_CALLBACK (window_cmd_file_bookmark_page), NULL },
 	{ "GoBookmarks", N_("_Edit Bookmarks"), EPHY_STOCK_BOOKMARKS, "<control>B",
-	  N_("Go to a bookmark"),
+	  N_("Open the bookmarks window"),
 	  G_CALLBACK (window_cmd_go_bookmarks), NULL },
 
 	/* Go menu */
@@ -180,7 +183,7 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	  N_("Go to a specified location"),
 	  G_CALLBACK (window_cmd_go_location), NULL },
 	{ "GoHistory", N_("H_istory"), EPHY_STOCK_HISTORY, "<control>H",
-	  N_("Go to an already visited page"),
+	  N_("Open the history window"),
 	  G_CALLBACK (window_cmd_go_history), NULL },
 
 	/* Tabs menu */
@@ -533,7 +536,8 @@ ephy_window_fullscreen (EphyWindow *window)
 
 	window->priv->is_fullscreen = TRUE;
 
-	tmodel = ephy_shell_get_toolbars_model (ephy_shell);
+	tmodel = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 	ephy_toolbars_model_set_flag (tmodel, EGG_TB_MODEL_ICONS_ONLY);
 
 	popup = gtk_window_new (GTK_WINDOW_POPUP);
@@ -576,7 +580,8 @@ ephy_window_unfullscreen (EphyWindow *window)
 
 	window->priv->is_fullscreen = FALSE;
 
-	tmodel = ephy_shell_get_toolbars_model (ephy_shell);
+	tmodel = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 	ephy_toolbars_model_unset_flag (tmodel, EGG_TB_MODEL_ICONS_ONLY);
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT (gdk_screen_get_default ()),
@@ -661,6 +666,8 @@ setup_window (EphyWindow *window)
 	g_object_set (action, "short_label", _("Print"), NULL);
 	action = egg_action_group_get_action (action_group, "FileBookmarkPage");
 	g_object_set (action, "short_label", _("Bookmark"), NULL);
+	action = egg_action_group_get_action (action_group, "EditFind");
+	g_object_set (action, "short_label", _("Find"), NULL);
 	action = egg_action_group_get_action (action_group, "GoBookmarks");
 	g_object_set (action, "short_label", _("Bookmarks"), NULL);
 
@@ -695,32 +702,6 @@ setup_window (EphyWindow *window)
 			  window);
 }
 
-static void
-sync_find_dialog (FindDialog *dialog, GParamSpec *spec, EphyWindow *window)
-{
-	EggActionGroup *action_group;
-	EggAction *action;
-	FindNavigationFlags flags;
-	gboolean can_go_prev = FALSE, can_go_next = FALSE;
-
-	flags = find_dialog_get_navigation_flags (dialog);
-
-	if (flags & FIND_CAN_GO_PREV)
-	{
-		can_go_prev = TRUE;
-	}
-	if (flags & FIND_CAN_GO_NEXT)
-	{
-		can_go_next = TRUE;
-	}
-
-	action_group = window->priv->action_group;
-	action = egg_action_group_get_action (action_group, "EditFindPrev");
-	g_object_set (action, "sensitive", can_go_prev, NULL);
-	action = egg_action_group_get_action (action_group, "EditFindNext");
-	g_object_set (action, "sensitive", can_go_next, NULL);
-}
-
 static void
 sync_tab_address (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 {
@@ -747,8 +728,9 @@ sync_tab_icon (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 
 	if (window->priv->closing) return;
 
-	cache = ephy_embed_shell_get_favicon_cache
-			(EPHY_EMBED_SHELL (ephy_shell));
+	cache = EPHY_FAVICON_CACHE
+		(ephy_embed_shell_get_favicon_cache
+			(EPHY_EMBED_SHELL (ephy_shell)));
 
 	address = ephy_tab_get_icon_address (tab);
 
@@ -777,7 +759,7 @@ sync_tab_load_progress (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 }
 
 static void
-sync_tab_load_status (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
+sync_tab_load_status (EphyTab *dummy, GParamSpec *pspec, EphyWindow *window)
 {
 	gboolean spin = FALSE;
 	GList *tabs, *l;
@@ -1325,6 +1307,8 @@ tab_removed_cb (EphyNotebook *notebook, GtkWidget *child, EphyWindow *window)
 
 	window->priv->num_tabs--;
 
+	sync_tab_load_status (NULL, NULL, window);
+
 	if (window->priv->num_tabs == 0)
 	{
 		/* removed the last tab, close the window */
@@ -1394,7 +1378,7 @@ ephy_window_init (EphyWindow *window)
 
 	LOG ("EphyWindow initialising %p", window)
 
-	session = ephy_shell_get_session (ephy_shell);
+	session = SESSION (ephy_shell_get_session (ephy_shell));
 
         window->priv = g_new0 (EphyWindowPrivate, 1);
 	window->priv->active_tab = NULL;
@@ -1455,7 +1439,7 @@ remove_from_session (EphyWindow *window)
 {
 	Session *session;
 
-	session = ephy_shell_get_session (ephy_shell);
+	session = SESSION (ephy_shell_get_session (ephy_shell));
 	g_return_if_fail (session != NULL);
 
 	session_remove_window (session, window);
@@ -1755,7 +1739,6 @@ EphyTab *
 ephy_window_get_active_tab (EphyWindow *window)
 {
 	g_return_val_if_fail (IS_EPHY_WINDOW (window), NULL);
-	g_return_val_if_fail (window->priv->active_tab != NULL, NULL);
 
 	return window->priv->active_tab;
 }
@@ -1846,31 +1829,26 @@ ephy_window_notebook_switch_page_cb (GtkNotebook *notebook,
 	update_tabs_menu_sensitivity (window);
 }
 
-EphyDialog *
-ephy_window_get_find_dialog (EphyWindow *window)
+void
+ephy_window_find (EphyWindow *window)
 {
 	EphyDialog *dialog;
 	EphyEmbed *embed;
 
-	if (window->priv->find_dialog)
+	if (window->priv->find_dialog == NULL)
 	{
-		return window->priv->find_dialog;
+		embed = ephy_window_get_active_embed (window);
+		g_return_if_fail (GTK_IS_WINDOW(window));
+
+		dialog = find_dialog_new_with_parent (GTK_WIDGET(window),
+						      embed);
+		window->priv->find_dialog = dialog;
+
+		g_object_add_weak_pointer(G_OBJECT (dialog),
+					  (gpointer *) &window->priv->find_dialog);
 	}
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_val_if_fail (GTK_IS_WINDOW(window), NULL);
-	dialog = find_dialog_new_with_parent (GTK_WIDGET(window),
-					      embed);
-
-	sync_find_dialog (FIND_DIALOG (dialog), NULL, window);
-	g_signal_connect_object (dialog, "notify::embed",
-				 G_CALLBACK (sync_find_dialog), window, 0);
-	g_signal_connect_object (dialog, "notify::navigation",
-				 G_CALLBACK (sync_find_dialog), window, 0);
-
-	window->priv->find_dialog = dialog;
-
-	return dialog;
+	ephy_dialog_show (window->priv->find_dialog);
 }
 
 static void
