@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.69 2003/06/27 19:09:51 mpeseng Exp $
+ *  $Id: window-commands.c,v 1.73 2003/07/20 11:05:16 mpeseng Exp $
  */
 
 #include <config.h>
@@ -23,7 +23,6 @@
 #include "ephy-shell.h"
 #include "ephy-debug.h"
 #include "window-commands.h"
-#include "find-dialog.h"
 #include "print-dialog.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
@@ -39,6 +38,8 @@
 #include "ephy-state.h"
 #include "ephy-gui.h"
 #include "ephy-zoom.h"
+#include "prefs-dialog.h"
+#include "ephy-toolbars-model.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -46,6 +47,7 @@
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-stock-icons.h>
+#include <libgnomeui/gnome-icon-theme.h>
 #include <libgnome/gnome-program.h>
 #include <gtk/gtkeditable.h>
 
@@ -58,12 +60,7 @@ void
 window_cmd_edit_find (EggAction *action,
 		      EphyWindow *window)
 {
-	EphyDialog *dialog;
-	dialog = ephy_window_get_find_dialog (window);
-
-	g_object_ref (dialog);
-
-	ephy_dialog_show (dialog);
+	ephy_window_find (window);
 }
 
 void
@@ -324,7 +321,7 @@ window_cmd_file_open (EggAction *action,
 
 	result = ephy_embed_single_show_file_picker
 		(single, wmain,
-		 _("Select the file to open"),
+		 _("Open"),
                  dir, NULL, modeOpen,
                  &file, NULL, NULL);
 
@@ -365,6 +362,7 @@ window_cmd_file_save_as (EggAction *action,
 				      EMBED_PERSIST_MAINDOC);
 
 	ephy_embed_utils_save (GTK_WIDGET(window),
+			       _("Save As"),
 			       CONF_STATE_SAVE_DIR,
 			       TRUE,
 			       TRUE,
@@ -472,22 +470,24 @@ void
 window_cmd_edit_find_next (EggAction *action,
 			   EphyWindow *window)
 {
-	EphyDialog *dialog;
+	EphyEmbed *embed;
 
-	dialog = ephy_window_get_find_dialog (window);
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	find_dialog_go_next (FIND_DIALOG(dialog), FALSE);
+	ephy_embed_find_next (embed, FALSE);
 }
 
 void
 window_cmd_edit_find_prev (EggAction *action,
 			   EphyWindow *window)
 {
-	EphyDialog *dialog;
+	EphyEmbed *embed;
 
-	dialog = ephy_window_get_find_dialog (window);
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	find_dialog_go_prev (FIND_DIALOG(dialog), FALSE);
+	ephy_embed_find_next (embed, TRUE);
 }
 
 void
@@ -645,7 +645,8 @@ toolbar_editor_response_cb (GtkDialog  *dialog,
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		break;
 	case RESPONSE_ADD_TOOLBAR:
-		model = ephy_shell_get_toolbars_model (ephy_shell);
+		model = EPHY_TOOLBARS_MODEL
+			(ephy_shell_get_toolbars_model (ephy_shell));
 		n = egg_toolbars_model_n_toolbars (EGG_TOOLBARS_MODEL (model));
 		egg_toolbars_model_add_toolbar (EGG_TOOLBARS_MODEL (model),
 						n - 1, "UserCreated");
@@ -665,7 +666,8 @@ window_cmd_edit_toolbar (EggAction *action,
 	Toolbar *t;
 	GtkWidget *dialog;
 
-	model = ephy_shell_get_toolbars_model (ephy_shell);
+	model = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 	t = ephy_window_get_toolbar (window);
 
 	dialog = gtk_dialog_new ();
@@ -723,8 +725,9 @@ window_cmd_help_about (EggAction *action,
 	static GtkWidget *about = NULL;
 	GtkWidget** ptr;
 	GdkPixbuf *icon;
-	const char *icon_path;
+	char *icon_path;
 	GdkPixbuf *logo;
+	GnomeIconTheme *icon_theme;
 
 	static gchar *authors[] = {
 		"Marco Pesenti Gritti <mpeseng@tin.it>",
@@ -750,10 +753,21 @@ window_cmd_help_about (EggAction *action,
 		return;
 	}
 
-	icon_path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					  "epiphany.png", TRUE, NULL);
-	logo = gdk_pixbuf_new_from_file (icon_path, NULL);
-	g_return_if_fail (logo != NULL);
+	icon_theme = gnome_icon_theme_new ();
+	icon_path = gnome_icon_theme_lookup_icon (icon_theme, "web-browser",
+						  -1, NULL, NULL);
+	g_object_unref (icon_theme);
+
+	if (icon_path)
+	{
+		logo = gdk_pixbuf_new_from_file (icon_path, NULL);
+		g_free (icon_path);
+	}
+	else
+	{
+		logo = NULL;
+		g_warning ("Web browser gnome icon not found");
+	}
 
 	about = gnome_about_new(
 		       "Epiphany", VERSION,
@@ -768,11 +782,11 @@ window_cmd_help_about (EggAction *action,
 
 	gtk_window_set_transient_for (GTK_WINDOW (about),
 				      GTK_WINDOW (window));
-	
-	icon = gtk_widget_render_icon (about, 
-						      GNOME_STOCK_ABOUT,
-						      GTK_ICON_SIZE_MENU,
-						      NULL);
+
+	icon = gtk_widget_render_icon (about,
+				       GNOME_STOCK_ABOUT,
+				       GTK_ICON_SIZE_MENU,
+				       NULL);
 	gtk_window_set_icon (GTK_WINDOW (about), icon);
 	g_object_unref(icon);
 
